
#include <stdio.h>
#include <sys/select.h>
#include <pty.h>
#include <utmp.h>
#include <unistd.h>
#include <termios.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>

#define DATA_BEGIN (2)
#define DATA_END (5)

pid_t childpid ;

int not_done=(1==1) ;

void signal_int(int x)
{
  kill(childpid, SIGINT ) ;
}

void child_died(int x)
{
  not_done=(1!=1) ;
  exit(0) ;
}
 
int main()
{
  char buffer[4096], st[4096];
  char *wbuffer ;
  struct termios orig_termi ;
  int master ;

  int size_wbuffer, get_wbuffer=0, put_wbuffer=0 ;

  ssize_t nread;
  int i,j ;
  int ignore=0 ;

  char *TEXMACS_PATH, *TEXMACS_R, *TEXMACS_SEND ;
  struct termios termi ;

  sigset_t sigmask;

  wbuffer = malloc( 4096 ) ;
  size_wbuffer = 4096 ;
  
  sigemptyset (&sigmask);
  sigaddset (&sigmask, SIGCHLD);
  sigaddset (&sigmask, SIGINT);

  TEXMACS_PATH = getenv("TEXMACS_PATH") ;
  if( TEXMACS_PATH == NULL )
    TEXMACS_PATH = "/home/dirk/work"; 

  TEXMACS_R = getenv("TEXMACS_CMD") ;
  if( TEXMACS_R == NULL )
    TEXMACS_R = "R"; 

  TEXMACS_SEND = getenv("TEXMACS_SEND") ;
  if( TEXMACS_SEND == NULL ) {
    TEXMACS_SEND = malloc( 4096 ) ;
    snprintf(TEXMACS_SEND,4096,"source(\"%s/plugins/r/r/texmacs_init.R\")\n",TEXMACS_PATH) ;
  } 
  /* prepare for this string to be sent to the process */
  strncpy(wbuffer,TEXMACS_SEND,4096) ;
  get_wbuffer += strlen( TEXMACS_SEND ) ;
  /* ignore 1 input request - i.e. do not generate a prompt channel in
     texmacs for it */
  ignore=1 ;

  if( (childpid=forkpty( &master, NULL, NULL, NULL ))==0 ) {
    /* child */
    execlp(TEXMACS_R,TEXMACS_R,NULL) ;
  } else {
    /* parent */
    signal (SIGINT, signal_int);
    signal (SIGCHLD, child_died);


    /* get terminal settings */
    tcgetattr(master, &orig_termi ) ;
#if 0
    printf("%x %x %x %x\n",termi.c_iflag,termi.c_oflag,termi.c_lflag,termi.c_lflag&ICANON) ;
#endif
    orig_termi.c_lflag &= ~ECHO  ; /* no echo */
    /* set tserminal settings */
    tcsetattr(master,TCSANOW, &orig_termi ) ;

#if 0
    printf("%x %x %x %x\n",termi.c_iflag,termi.c_oflag,termi.c_lflag,termi.c_lflag&ICANON) ;
#endif
    

    
    write( STDOUT_FILENO, st, sprintf(st,"%cverbatim:",DATA_BEGIN) ) ;  /*  1 */
    
    while(not_done) {
      /* prepare the file sets for select to watch */
      fd_set rd, wr, er;
      FD_ZERO (&rd);
      FD_ZERO (&wr);
      FD_ZERO (&er);
      FD_SET (master, &rd);
      FD_SET (master, &wr);
      FD_SET (STDIN_FILENO, &rd ) ;
      if( pselect( master+1, &rd, &wr, &er, NULL, &sigmask ) > 0 ) {
	
	if( FD_ISSET( STDIN_FILENO, &rd ) ) {
	  /* read input from TeXmacs */
	  if( size_wbuffer - get_wbuffer < 1000 ) {
	    /* we don't have enough room left for more input */
	    size_wbuffer *= 2 ;
	    wbuffer = realloc( wbuffer, size_wbuffer ) ;
	  }
	  
	  nread = read( STDIN_FILENO, wbuffer+get_wbuffer, 1000 ) ;
	  for(i=0; i<nread-1; i++)
	    if( wbuffer[get_wbuffer+i]=='\n' )
	      wbuffer[get_wbuffer+i]=' ' ; /* change new-line to
					      space, so that it all
					      appears on one line -
					      except for the last
					      new-line */
	  get_wbuffer += nread ;
	}

	if( FD_ISSET( master, &rd ) && (nread = read( master, buffer, 4096 )) ) {
	  /* read input from sub process (R) */
	  tcgetattr(master, &termi ) ;
#if 0
	  printf("%x %x %x %x\n",termi.c_iflag,termi.c_oflag,termi.c_lflag,termi.c_lflag&ICANON) ;
#endif

	  /* remove \n (should be done by pty, but couldn't get that
	     to work) */
	  for(i=0,j=0; i<nread; i++,j++) {
	    if( i > j )
	      buffer[j] = buffer[i] ;
	    if( buffer[i]== 13 )
	      j-- ;
	  }
	  nread = j ;
	
	  
	  if( (termi.c_lflag & ICANON ) == 0 ) {
	    /* The terminal wants input from the user */
	    if(ignore>0)
	      ignore-- ; /* how many input requests should be ignored?
			  */
	    else {
	      write( STDOUT_FILENO, st, sprintf(st,"%cverbatim:",DATA_BEGIN) ) ; /* { 2 */
	      
	      /* find the previous end-of-line, and use that for the
		 prompt */
	      for( i=nread-1; i>=0; i--) 
		if( buffer[i]=='\n' )
		  break ;
	      
	      if( i > 0 ) {
		/* print everything before the pervious end-of-line */
		write( STDOUT_FILENO, buffer, i ) ;
	      }
	      i++ ; /* ignore the end-of-line itself */
	      if( ignore == 0 ) { /* we are dealing with this input */
		write( STDOUT_FILENO, st, sprintf(st,"%c",DATA_END) ) ;	  /*  2 } */ 
		write( STDOUT_FILENO, st, sprintf(st,"%cchannel:prompt%c",DATA_BEGIN,DATA_END) ) ; /* { 3 } */
		write( STDOUT_FILENO, st, sprintf(st,"%cverbatim:",DATA_BEGIN) ) ; /* { 2 */
	      }
	      
	      write( STDOUT_FILENO, buffer+i, nread-i ) ;
	      
	      write( STDOUT_FILENO, st, sprintf(st,"%c",DATA_END) ) ; /* 2 } */
	      write( STDOUT_FILENO, st, sprintf(st,"%c",DATA_END) ) ; /* 1 } */
	      write( STDOUT_FILENO, st, sprintf(st,"%cverbatim:",DATA_BEGIN) ) ; /* { 1 */
	    }
	  } else {
	    /* terminal is not waiting for user - just print data. */
	    write( STDOUT_FILENO, buffer, nread ) ;
	  }

      }
      if( FD_ISSET( master, &wr ) ) {
	/* Terminal is ready to receive data */
	if( get_wbuffer > put_wbuffer ) {
	  /* we have data available */
	  nread = write( master, wbuffer+put_wbuffer, get_wbuffer-put_wbuffer ) ;
	  put_wbuffer += nread ;
	  if( put_wbuffer == get_wbuffer ) 
	    get_wbuffer = put_wbuffer = 0 ; /* reset write buffer */
	}
      }

      /*tcsetattr(master,TCSANOW, &orig_termi ) ;*/

      }
    }
      
  }
  exit(0) ;
}
