
/******************************************************************************
* MODULE     : convert.hh
* DESCRIPTION: various conversion routines
* COPYRIGHT  : (C) 1999  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef CONVERT_H
#define CONVERT_H
#include "url.hh"
#include "analyze.hh"
#include "vars.hh"
typedef tree scheme_tree;

/*** Generic ***/
string get_file_format (url file_name);
string get_document_format (string s);
tree   generic_document_to_tree (string s, string format);
string tree_to_generic_document (tree doc, string format);

/*** Texmacs ***/
tree texmacs_to_tree (string s);
tree texmacs_document_to_tree (string s);
string tree_to_texmacs (tree t);
string tree_to_texmacs_document (tree t);
tree extract (tree doc, string attr);
tree extract_document (tree doc);
hashmap<string,int> get_codes (string version);
tree string_to_tree (string s, string version);
tree upgrade (tree t, string version);

/*** Scheme ***/
string scheme_tree_to_string (scheme_tree p);
string scheme_tree_to_block (scheme_tree p);
scheme_tree tree_to_scheme_tree (tree t);
string tree_to_scheme (tree t);
string tree_to_scheme_document (tree t);
scheme_tree string_to_scheme_tree (string s);
scheme_tree block_to_scheme_tree  (string s);
tree scheme_tree_to_tree (scheme_tree p);
tree scheme_to_tree (string s);
tree scheme_document_to_tree (string s);

/*** Verbatim ***/
string tree_to_verbatim (tree t, bool pritty= false);
tree   verbatim_to_tree (string s);

/*** Tex ***/
string tree_to_tex (tree t, string mode);
string tree_to_tex_document (tree t, tree style, string lan);
string tree_to_latex (tree t, string mode);
string tree_to_latex_document (tree t, tree style, string lan);
tree   tex_to_tree (string s, string mode);
tree   tex_document_to_tree (string s, string& style);
tree   latex_to_tree (string s, string mode);
tree   latex_document_to_tree (string s, string& style);

/*** Html ***/
string tree_to_html (tree t);
string tree_to_html_document (tree t, tree style, string lan);
tree   parse_html (string s);
tree   html_to_tree (string s, string mode);
tree   html_document_to_tree (string s, string& style);

/*** Lisp (obsolete) ***/
tree   dl_compile_lisp (tree t);
tree   dl_eval_lisp (tree t);

#endif // defined CONVERT_H
